/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Helmholtz Centre for Environmental Research - UFZ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 * main.c
 *
 * This software runs the Raincollector MLM 1.0.
 *
 * This program is purposed to run on a Texas Instruments EXP430FR6989 LaunchPad
 * Development Board with a Texas Instruments BOOST-DRV8711 attached.
 *
 *  Created on: 07.02.2017
 *      Author: G. Laube
 */

#include <driverlib.h>  // MSP430 module functions provided by TI
#include <stdio.h>
#include "hal_LCD.h"    // hardware abstraction layer for LCD
#include "timer_mode.h"    // hardware abstraction layer for RTC
#include "hal_DRV8711.h" // hardware abstraction layer for stepper driver

/* Main menu sections:
 * - first: "reference" the motor position,
 * - second: decide between timer or counter mode in "startup" mode
 * - third: either "timer" or "counter" mode
*/
#define REFERENCE_MODE      0
#define STARTUP_MODE        2
#define TIMER_MODE          4
#define COUNTER_MODE        6

// Menu sub-sections: different menu definition within each mode
// sub menus of "REFERENCE_MODE"
#define REF_STARTUP_MENU    0
#define REF_TURN_MENU       2

// sub menus of "STARTUP_MODE"
#define STARTUP_MENU        0

// sub menus of "TIMER_MODE"
#define TIMER_STARTUP_MENU   0
#define TIMER_DAY_MENU       1
#define TIMER_HOUR_MENU      2
#define TIMER_MINUTE_MENU    3
#define TIMER_ST_SCROLLTEXT  4
#define TIMER_ST_DAY_MENU    5
#define TIMER_ST_HOUR_MENU   6
#define TIMER_ST_MINUTE_MENU 7
#define TIMER_RUN_MENU       8

// start in reference mode to manually turn the motor to "position 0"
volatile unsigned char mode = REFERENCE_MODE;
volatile unsigned char menu = REF_STARTUP_MENU;

// initialize variables for selections via rotary encoder
volatile int counter = 0;
volatile char unit = ' ';

// TimerA0 UpMode Configuration Parameter for debouncing buttons
Timer_A_initUpModeParam initUpParam_A0 =
{
        TIMER_A_CLOCKSOURCE_ACLK,               // ACLK Clock Source
        TIMER_A_CLOCKSOURCE_DIVIDER_1,          // ACLK = 32768
        3277,                                   // 100ms debounce period
        TIMER_A_TAIE_INTERRUPT_DISABLE,         // Disable Timer interrupt
        TIMER_A_CCIE_CCR0_INTERRUPT_ENABLE ,    // Enable CCR0 interrupt
        TIMER_A_DO_CLEAR,                       // Clear value
        true                                    // Start Timer
};

void init_GPIO(void);
void init_Clock(void);

void main(void)
{
    // Stop watchdog timer
    WDT_A_hold(WDT_A_BASE);

    // Initializations
    init_GPIO();
    init_Clock();
    Init_LCD();
    init_DRV8711();

    // enable general interrupts
    __enable_interrupt();


    displayContinuousScrollText("TURN WHEEL TO DRIVE MOTOR TO REFERENCE POSITION      PRESS RIGHT TO START AND AGAIN TO CONFIRM");

    GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN1|GPIO_PIN2);

    while(1)
    {
        __bis_SR_register(LPM3_bits | GIE);         // enter LPM3
    }
}

void init_GPIO()
{
    /* ESI Scan Pinout
     *
     * ---left--- S1, LED1
     *
     * GND      -x  x- ?
     * GND      -x  x- 3.3
     * ESICOM   -x  x- ESICI
     * (P9.7)   -x  x- P9.6   -> used for LED2
     * P9.5     -x  x- P9.4
     * (P9.3)   -x  x- (P9.2) -> used for BOOST-DRV87711 nSLEEP -x x- POT
     * P9.1     -x  x- P9.0
     * GND      -x  x- P1.3
     *
     * ---right--- S2, LED2
     * */

    /* BOOST-DRV8711 Pinout
     *
     *  3.3     -x x- GND
     *  P9.2    -x x- P2.1
     *  -       -x x- P1.5
     *  -       -x x- -
     *  -       -x x- -
     *  9.3     -x x- 1.6
     *  1.4     -x x- 1.7
     *  2.0     -x x- 2.5
     *  4.1     -x x- 2.4
     *  4.0     -x x- 4.7
     */

    /*
     * Rotary Encoder
     *
     * P2.7     -x- B
     * P2.6     -x- Common
     * P3.3     -x- A
     */

    // Disable the GPIO power-on default high-impedance mode
    // looks like this need to happen before enabling the button interrupts
    // weird interrupts occur if this happens afterwards
    PMM_unlockLPM5();

    // Set all GPIO pins to output low to prevent floating input and reduce power consumption
    // All PxOUT are undefined upon reset
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P5, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P6, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P7, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P8, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P9, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);

    // All ports are inputs upon reset
    GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P5, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P6, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P7, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P8, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P9, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);

    // Set DRV8711 nFault (P1.5), nStall (P2.1) and Pot (P9.2) as input
    // Pin as output low would cause the fault-LED to be always on
    // Pin as output high would cause a shortcut if fault occurs
    // Pin 9.2 is connected to either 3.3V or GND, depending on potentiometer position
    GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN5);
    GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P2, GPIO_PIN1);
    GPIO_setAsInputPin(GPIO_PORT_P9, GPIO_PIN2);

    // Configure button S1 (P1.1) interrupt, not enabled yet
    GPIO_selectInterruptEdge(GPIO_PORT_P1, GPIO_PIN1, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN1);
    GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN1);

    // Configure button S2 (P1.2) interrupt, not enabled yet
    GPIO_selectInterruptEdge(GPIO_PORT_P1, GPIO_PIN2, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN2);
    GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN2);

    // Configure rotary encoder Channel A (P2.7) interrupt, not enabled yet
    GPIO_selectInterruptEdge(GPIO_PORT_P2, GPIO_PIN7, GPIO_LOW_TO_HIGH_TRANSITION);
    GPIO_setAsInputPinWithPullDownResistor(GPIO_PORT_P2, GPIO_PIN7);
    GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN7);

    // Configure rotary encoder Channel B (P3.3) interrupt, not enabled yet
    GPIO_selectInterruptEdge(GPIO_PORT_P3, GPIO_PIN3, GPIO_LOW_TO_HIGH_TRANSITION);
    GPIO_setAsInputPinWithPullDownResistor(GPIO_PORT_P3, GPIO_PIN3);
    GPIO_clearInterrupt(GPIO_PORT_P3, GPIO_PIN3);

    // Set PJ.4 and PJ.5 as Secondary Module Function Input, LFXT.
    // According to Device Data Sheet p120, PJSELy.5 & PJDIRx is "don't care"
    GPIO_setAsPeripheralModuleFunctionInputPin(
           GPIO_PORT_PJ,
           GPIO_PIN4,
           GPIO_PRIMARY_MODULE_FUNCTION
           );

    // Select P1.4 to input Secondary Module Function, (UCB0CLK)
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P1,
        GPIO_PIN4,
        GPIO_PRIMARY_MODULE_FUNCTION
        );

    // Select 1.6 & 1.7 to input Secondary Module Function, (UCB0TXD/UCB0SIMO, UCB0RXD/UCB0SOMI)
    GPIO_setAsPeripheralModuleFunctionInputPin(
       GPIO_PORT_P1,
       GPIO_PIN6 + GPIO_PIN7,
       GPIO_PRIMARY_MODULE_FUNCTION
       );
}

void init_Clock()
{
    // Set DCO frequency to default 1MHz
    CS_setDCOFreq(CS_DCORSEL_0, CS_DCOFSEL_0);

    // Configure MCLK and SMCLK to default 1MHz from DCO
    CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);

    // Initializes the XT1 crystal oscillator with full drive current
    // (taken from launch pad OutOfBox-Software, not tested if CS_LFXT_DRIVE_3 is necessary)
    // turned on without timeout to let it settle because a stable signal is not needed early
    CS_turnOnLFXT(CS_LFXT_DRIVE_3);

    // Configure ACLK to default 32768Hz
    CS_initClockSignal(CS_ACLK, CS_LFXTCLK_SELECT, CS_CLOCK_DIVIDER_1);
}

/*
 * PORT1 Interrupt Service Routine
 * Handles S1 and S2 button press interrupts
*/
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
    /*
     * Set a common debounce flag on first high to low transition of either button
     * A common debounce flag is good enough because
     * buttons will not be pushed shortly after another
     */
    // copy current interrupt flag register, just in case it will be needed after it changed during debounce period
    unsigned short P1IV_CPY = P1IV;
    GPIO_disableInterrupt(GPIO_PORT_P1, GPIO_PIN1|GPIO_PIN2);
    // interrupt will be reenabled by TimerA0 after 15ms after button was released
    Timer_A_initUpMode(TIMER_A0_BASE, &initUpParam_A0);

    if (mode == REFERENCE_MODE)
    {
        // if button 2 was pressed (button 1 actually not used in reference mode)
        if  (P1IV_CPY == P1IV_P1IFG2)
        {
            switch(menu)
            {
            case REF_STARTUP_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = REF_TURN_MENU;  // proceed from startup scrolltext to day selection
                stopScrollText();

                // enable rotary encoder
                GPIO_enableInterrupt(GPIO_PORT_P2, GPIO_PIN7);
                GPIO_enableInterrupt(GPIO_PORT_P3, GPIO_PIN3);
                GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN6);
                break;

            case REF_TURN_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                mode = TIMER_MODE;
                menu = TIMER_STARTUP_MENU;
                displayContinuousScrollText("CHOOSE SAMPLING INTERVAL      PRESS RIGHT TO START");
                break;

            default: break;
            }
        }
    }

// not used in this version. Will be used for precipitation-driven turning
/*
    else if (mode == STARTUP_MODE)
    {
        // switch is more efficient for interrupt handling, if used with __even_in_range
        // switch if button 1 or button 2 was pressed
        switch(__even_in_range(P1IV_CPY, P1IV_P1IFG7))
        {
        case P1IV_P1IFG1 :    // Button S1 pressed
            GPIO_setOutputHighOnPin(GPIO_PORT_P1, GPIO_PIN0);    // Turn LED1 On
            mode = TIMER_MODE;
            menu = TIMER_STARTUP_MENU;
            stopScrollText();
            displayContinuousScrollText("TIME MODE      CHOOSE SAMPLING INTERVAL      PRESS RIGHT TO START");
            break;
        case P1IV_P1IFG2 :    // Button S2 pressed
            GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
            mode = COUNTER_MODE;
            stopScrollText();
            displayContinuousScrollText("PRECIPITATION MODE      CHOOSE SAMPLING INTERVAL IN CLICKS      PRESS RIGHT TO START");
            break;
        default : break;
        }
    }
*/

    else if (mode == TIMER_MODE)
    {
        // if button 2 was pressed (button 1 actually not used in timer mode)
        if  (P1IV_CPY == P1IV_P1IFG2)
        {
            switch(menu)
            {
            case TIMER_STARTUP_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_DAY_MENU;  // proceed from startup scrolltext to day selection
                stopScrollText();
                counter = 0;
                unit = 'D';
                showFourDigitValueAndUnit(counter%4, unit);
                GPIO_enableInterrupt(GPIO_PORT_P2, GPIO_PIN7);
                GPIO_enableInterrupt(GPIO_PORT_P3, GPIO_PIN3);
                GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN6);
                break;

            case TIMER_DAY_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_HOUR_MENU; // proceed from day to hour selection
                days = counter / 4;
                counter = 0;
                unit = 'H';
                showFourDigitValueAndUnit(counter%4, unit);
                break;

            case TIMER_HOUR_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_MINUTE_MENU; // proceed from hour to minute selection
                hours = counter / 4;
                counter = 0;
                unit = 'M';
                showFourDigitValueAndUnit(counter%4, unit);
                break;

            case TIMER_MINUTE_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_ST_SCROLLTEXT; // proceed from minute to start time scrolltext
                minutes = counter / 4;
                displayContinuousScrollText("CHOOSE TIME TO START      PRESS RIGHT TO START");
                break;

            case TIMER_ST_SCROLLTEXT:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_ST_DAY_MENU; // proceed from minute to start in days selection
                stopScrollText();
                counter = 0;
                unit = 'D';
                showFourDigitValueAndUnit(counter%4, unit);
                break;

            case TIMER_ST_DAY_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_ST_HOUR_MENU; // proceed from start in days to start in hours selection
                st_days = counter / 4;
                counter = 0;
                unit = 'H';
                showFourDigitValueAndUnit(counter%4, unit);
                break;

            case TIMER_ST_HOUR_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_ST_MINUTE_MENU; // proceed from start in hours to start in minutes selection
                st_hours = counter / 4;
                counter = 0;
                unit = 'M';
                showFourDigitValueAndUnit(counter%4, unit);
                break;

            case TIMER_ST_MINUTE_MENU:
                GPIO_setOutputHighOnPin(GPIO_PORT_P9, GPIO_PIN7);    // Turn LED2 On
                menu = TIMER_RUN_MENU; // End selection, start runtime
                st_minutes = counter / 4;
                counter = 0;
                char sleepText[74];
                sprintf (sleepText, "SAMPLING INTERVAL %2uD %2uH %2uM STARTING IN %2uD %2uH %2uM      ENTER SLEEP MODE", days, hours, minutes, st_days, st_hours, st_minutes);
                displayScrollText(sleepText);
                clearLCD();
                LCD_C_off(LCD_C_BASE);
                init_RTC();
                break;

            default : break;
            }
        }
    }
}

// ISR for Channel A of rotary encoder
#pragma vector = PORT3_VECTOR
__interrupt void PORT3_ISR(void)
{
    /* Gray Code Table of Rotary Encoder Bourns ECW1J-B24-AC0024L
     *
     * Clockwise ->  _ _     _ _     _ _
     * Channel A:  _|   |_ _|   |_ _|   |_ _|
     *                 _ _     _ _     _ _
     * Channel B:  _ _|   |_ _|   |_ _|   |_ _|
     * Detent:     D       D       D       D
     * counter   = 0 1 2 3 4 5 6 7 8 9 ...
     * counter%4 = 0 1 2 3 0 1 2 3 0 1 ...
     * ISR A     =  I   I   I   I   I   I   I
     * ISR B     =    I   I   I   I   I   I   I
     * counter/4 = 0       1       2       3
     */
    switch((counter % 4 + 4) % 4)
    {
        case 0 : // detent
            // Channel A changes state from detent only if turned clockwise
            counter++;
            GPIO_selectInterruptEdge(GPIO_PORT_P3, GPIO_PIN3, GPIO_HIGH_TO_LOW_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P3, GPIO_PIN3);
            break;
        case 1 : // detent + 1
            // Channel A changes state from detent+1 only if turned anti-clockwise
            counter--; // no need to check if counter would get negative, because counter%4 = 1
            GPIO_selectInterruptEdge(GPIO_PORT_P3, GPIO_PIN3, GPIO_LOW_TO_HIGH_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P3, GPIO_PIN3);
            if (mode == REFERENCE_MODE) DRV8711_decrementStep();
            else showFourDigitValueAndUnit(counter/4, unit);
            break;
        case 2 : // detent + 2
            // Channel A changes state from detent+2 only if turned clockwise
            counter++;
            GPIO_selectInterruptEdge(GPIO_PORT_P3, GPIO_PIN3, GPIO_LOW_TO_HIGH_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P3, GPIO_PIN3);
            break;
        case 3 : // detent + 3
            // Channel A changes state from detent+3 only if turned anti-clockwise
            counter--; // no need to check if counter would get negative, because counter%4 = 3
            GPIO_selectInterruptEdge(GPIO_PORT_P3, GPIO_PIN3, GPIO_HIGH_TO_LOW_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P3, GPIO_PIN3);
            break;
        default : _never_executed();
    }
}

// ISR for Channel B of rotary encoder
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR(void)
{
    switch((counter % 4 + 4) % 4)
    {
        case 0 : // detent
            // Channel B changes state from detent only if turned anti-clockwise
            // negative values only allowed for driving the motor backwards to a reference position
            if (mode == REFERENCE_MODE) counter--;
            else if(counter != 0) counter--;
            GPIO_selectInterruptEdge(GPIO_PORT_P2, GPIO_PIN7, GPIO_HIGH_TO_LOW_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN7);
            break;
        case 1 : // detent + 1
            // Channel B changes state from detent+1 only if turned clockwise
            counter++;
            GPIO_selectInterruptEdge(GPIO_PORT_P2, GPIO_PIN7, GPIO_HIGH_TO_LOW_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN7);
            break;
        case 2 : // detent + 2
            // Channel B changes state from detent+2 only if turned anti-clockwise
            counter--; // no need to check if counter would get negative, because counter%4 = 2
            GPIO_selectInterruptEdge(GPIO_PORT_P2, GPIO_PIN7, GPIO_LOW_TO_HIGH_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN7);
            break;
        case 3 : // detent + 3
            // Channel B changes state from detent+3 only if turned clockwise
            if (mode == TIMER_MODE) {
                switch (menu)
                {
                    case TIMER_DAY_MENU :
                        if (counter < 30*4) counter++;
                        else counter -= 3;
                        break;
                    case TIMER_HOUR_MENU :
                        if (counter < 23*4) counter++;
                        else counter -= 3;
                        break;
                    case TIMER_MINUTE_MENU :
                        if (counter < 59*4) counter++;
                        else counter -= 3;
                        break;
                    case TIMER_ST_DAY_MENU :
                        if (counter < 30*4) counter++;
                        else counter -= 3;
                        break;
                    case TIMER_ST_HOUR_MENU :
                        if (counter < 23*4) counter++;
                        else counter -= 3;
                        break;
                    case TIMER_ST_MINUTE_MENU :
                        if (counter < 59*4) counter++;
                        else counter -= 3;
                        break;
                }
            }
            else counter++;
            GPIO_selectInterruptEdge(GPIO_PORT_P2, GPIO_PIN7, GPIO_LOW_TO_HIGH_TRANSITION);
            GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN7);
            if (mode == REFERENCE_MODE) DRV8711_incrementStep();
            else showFourDigitValueAndUnit(counter/4, unit);
            break;
        default : _never_executed();
    }
}

/*
 * Timer A0 Interrupt Service Routine
 * Used as button debounce timer
*/
#pragma vector = TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR (void)
{
    // Debounce only, if both were released to avoid interrupt from release bounce
    // Both buttons are pulled up, so proceed only if both == 1
    if ( GPIO_getInputPinValue(GPIO_PORT_P1, GPIO_PIN1) && GPIO_getInputPinValue(GPIO_PORT_P1, GPIO_PIN2))
    {
        GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN1|GPIO_PIN2);
        GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN1|GPIO_PIN2);
        GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0);
        GPIO_setOutputLowOnPin(GPIO_PORT_P9, GPIO_PIN7);

        Timer_A_stop(TIMER_A0_BASE);
    }

}


